Package: gemma.R
Title: A wrapper for Gemma's Restful API to access curated gene
        expression data and differential expression analyses
Version: 3.9.1
Authors@R: 
    c(person(given = "Javier", family = "Castillo-Arnemann", 
             role = c("aut"), email = "javiercastilloar@gmail.com",
             comment = c(ORCID = "0000-0002-5626-9004")),
      person(given = "Jordan", family = "Sicherman", 
             role = c("aut"), email = "jordan.sicherman@msl.ubc.ca",
             comment = c(ORCID = "0000-0001-8160-4567")),
      person(given = "Ogan", family = "Mancarci", 
             role = c("aut"), email = "ogan.mancarci@gmail.com",
             comment = c(ORCID = "0000-0002-1452-0889")),
      person(give = "Guillaume", family = "Poirier-Morency",
             role = c("aut"), email = "poirigui@msl.ubc.ca",
             comment = c(ORCID = "0000-0002-6554-0441")),
      person(give='Paul',family = 'Pavlidis',
             role=c("aut","cre"),email = "paul@msl.ubc.ca",
             comment = c(ORCID = "0000-0002-0426-5028")))
Description: Low- and high-level wrappers for Gemma's RESTful API. They
        enable access to curated expression and differential expression
        data from over 10,000 published studies. Gemma is a web site,
        database and a set of tools for the meta-analysis, re-use and
        sharing of genomics data, currently primarily targeted at the
        analysis of gene expression profiles.
URL: https://pavlidislab.github.io/gemma.R/,
        https://github.com/PavlidisLab/gemma.R
License: Apache License (>= 2)
Encoding: UTF-8
Roxygen: list(markdown = TRUE)
BugReports: https://github.com/PavlidisLab/gemma.R/issues
Depends: R (>= 4.1.0)
Imports: magrittr, glue, memoise, jsonlite, data.table, rlang,
        lubridate, utils, stringr, SummarizedExperiment, Biobase,
        tibble, tidyr, S4Vectors, httr, rappdirs, bit64, assertthat,
        digest, R.utils, kableExtra, base64enc, pheatmap, grDevices
Suggests: testthat (>= 2.0.0), rmarkdown, knitr, dplyr, covr, ggplot2,
        ggrepel, BiocStyle, microbenchmark, magick, purrr, viridis,
        poolr, listviewer, shiny
Config/testthat/edition: 2
VignetteBuilder: knitr
biocViews: Software, DataImport, Microarray, SingleCell,
        ThirdPartyClient, DifferentialExpression, GeneExpression,
        Bayesian, Annotation, ExperimentalDesign, Normalization,
        BatchEffect, Preprocessing
Config/roxygen2/version: 8.0.0
Config/pak/sysreqs: cmake libfontconfig1-dev libfreetype6-dev
        libfribidi-dev make libharfbuzz-dev libicu-dev libpng-dev
        libuv1-dev libxml2-dev libssl-dev zlib1g-dev
Repository: https://bioc.r-universe.dev
Date/Publication: 2026-06-01 22:18:05 UTC
RemoteUrl: https://github.com/bioc/gemma.R
RemoteRef: HEAD
RemoteSha: a359de629f19c6e7e8a83bc6fa1b5ee055d0c74e
NeedsCompilation: no
Packaged: 2026-06-02 18:50:16 UTC; root
Author: Javier Castillo-Arnemann [aut] (ORCID:
    <https://orcid.org/0000-0002-5626-9004>),
  Jordan Sicherman [aut] (ORCID: <https://orcid.org/0000-0001-8160-4567>),
  Ogan Mancarci [aut] (ORCID: <https://orcid.org/0000-0002-1452-0889>),
  Guillaume Poirier-Morency [aut] (ORCID:
    <https://orcid.org/0000-0002-6554-0441>),
  Paul Pavlidis [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-0426-5028>)
Maintainer: Paul Pavlidis <paul@msl.ubc.ca>
Built: R 4.6.0; ; 2026-06-02 18:55:33 UTC; unix
