# Generated by roxygen2: do not edit by hand

S3method(print,listable_pheatmap)
export(filter_properties)
export(forget_gemma_memoised)
export(gemma_call)
export(gemma_kable)
export(gemma_memoise)
export(get_all_pages)
export(get_annotation_children)
export(get_annotation_parents)
export(get_child_terms)
export(get_dataset_annotations)
export(get_dataset_differential_expression_analyses)
export(get_dataset_expression)
export(get_dataset_expression_for_genes)
export(get_dataset_object)
export(get_dataset_platforms)
export(get_dataset_processed_expression)
export(get_dataset_publications)
export(get_dataset_quantitation_types)
export(get_dataset_raw_expression)
export(get_dataset_samples)
export(get_datasets)
export(get_datasets_by_ids)
export(get_differential_expression_values)
export(get_gene_differential_expression_values)
export(get_gene_go_terms)
export(get_gene_locations)
export(get_gene_probes)
export(get_genes)
export(get_platform_annotations)
export(get_platform_datasets)
export(get_platform_element_genes)
export(get_platforms_by_ids)
export(get_result_sets)
export(get_taxa)
export(get_taxon_datasets)
export(make_design)
export(search_annotations)
export(search_datasets)
export(search_gemma)
export(set_gemma_user)
export(update_result)
export(visualize_dataset)
import(bit64)
import(data.table)
import(digest)
importFrom(magrittr,"%>%")
importFrom(rlang,.data)
